@@ -3,7 +3,7 @@
 **
 ** <file/class description>
 **
-** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
+** Copyright (C) 2002-2004 Steve Lhomme.  All rights reserved.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -30,13 +30,12 @@
 
 /*!
 	\file
-	\version \$Id: EbmlElement.cpp,v 1.12 2003/11/03 22:14:27 robux4 Exp $
+	\version \$Id: EbmlElement.cpp,v 1.16 2004/01/10 15:31:03 robux4 Exp $
 	\author Steve Lhomme     <robux4 @ users.sf.net>
 */
 
 #include <cassert>
 
-#include "StdInclude.h"
 #include "ebml/EbmlElement.h"
 #include "ebml/EbmlMaster.h"
 #include "ebml/EbmlStream.h"
@@ -51,7 +50,7 @@ START_LIBEBML_NAMESPACE
 */
 int CodedSizeLength(uint64 Length, unsigned int SizeLength)
 {
-	int CodedSize;
+	unsigned int CodedSize;
 	// prepare the head of the size (000...01xxxxxx)
 	// optimal size
 	if (Length < 127) // 2^7 - 1
@@ -77,7 +76,7 @@ int CodedSizeLength(uint64 Length, unsigned int SizeLength)
 */
 int CodedSizeLengthSigned(int64 Length, unsigned int SizeLength)
 {
-	int CodedSize;
+	unsigned int CodedSize;
 	// prepare the head of the size (000...01xxxxxx)
 	// optimal size
 	if (Length > -64 && Length < 64) // 2^6
@@ -130,8 +129,7 @@ uint64 ReadCodedSizeValue(const binary * InBuffer, uint32 & BufferSize, uint64 &
 {
 	binary SizeBitMask = 1 << 7;
 	uint64 Result = 0x7F;
-	int SizeIdx;
-	unsigned int PossibleSizeLength = 0;
+	unsigned int SizeIdx, PossibleSizeLength = 0;
 	binary PossibleSize[8];
 
 	SizeUnknown = 0x7F; // the last bit is discarded when computing the size
@@ -317,7 +315,7 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
 EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSemanticContext & Context, int & UpperLevel, 
 			uint64 MaxDataSize, bool AllowDummyElt, unsigned int MaxLowerLevel)
 {
-	int PossibleID_Length;
+	int PossibleID_Length = 0;
 	binary PossibleIdNSize[16];
 	int PossibleSizeLength;
 	uint64 SizeUnknown;
@@ -397,7 +395,7 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
 							Result->SetSizeInfinite();
 						}
 
-						Result->SizePosition = DataStream.getFilePointer() - _SizeLength;
+						Result->SizePosition = DataStream.getFilePointer() - SizeIdx + PossibleID.Length;
 						Result->ElementPosition = Result->SizePosition - PossibleID.Length;
 						// place the file at the beggining of the data
 						DataStream.setFilePointer(Result->SizePosition + _SizeLength);
