@@ -3,7 +3,7 @@
 **
 ** <file/class description>
 **
-** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
+** Copyright (C) 2002-2004 Steve Lhomme.  All rights reserved.
 **
 ** This file is part of libebml.
 **
@@ -32,14 +32,13 @@
 
 /*!
 	\file
-	\version \$Id: EbmlMaster.cpp,v 1.12 2003/11/03 22:14:27 robux4 Exp $
+	\version \$Id: EbmlMaster.cpp,v 1.16 2004/01/17 16:32:49 robux4 Exp $
 	\author Steve Lhomme     <robux4 @ users.sf.net>
 */
 
 #include <cassert>
 #include <algorithm>
 
-#include "StdInclude.h"
 #include "ebml/EbmlMaster.h"
 #include "ebml/EbmlStream.h"
 #include "ebml/EbmlContexts.h"
@@ -159,9 +158,9 @@ uint64 EbmlMaster::UpdateSize(bool bSaveDefault, bool bForceRender)
 	return Size;
 }
 
-uint32 EbmlMaster::WriteHead(IOCallback & output, int SizeLength, bool bSaveDefault)
+uint32 EbmlMaster::WriteHead(IOCallback & output, int nSizeLength, bool bSaveDefault)
 {
-	SetSizeLength(SizeLength);
+	SetSizeLength(nSizeLength);
 	return RenderHead(output, false, bSaveDefault);
 }
 
@@ -397,7 +396,7 @@ void EbmlMaster::Sort()
 	\todo add an option to discard even unknown elements
 	\todo handle when a mandatory element is not found
 */
-void EbmlMaster::Read(EbmlStream & inDataStream, const EbmlSemanticContext & Context, int & UpperEltFound, EbmlElement * & FoundElt, bool AllowDummyElt, ScopeMode ReadFully)
+void EbmlMaster::Read(EbmlStream & inDataStream, const EbmlSemanticContext & sContext, int & UpperEltFound, EbmlElement * & FoundElt, bool AllowDummyElt, ScopeMode ReadFully)
 {
 	if (ReadFully != SCOPE_NO_DATA)
 	{
@@ -414,11 +413,12 @@ void EbmlMaster::Read(EbmlStream & inDataStream, const EbmlSemanticContext & Con
 
 		// read blocks and discard the ones we don't care about
 		if (MaxSizeToRead > 0) {
-			ElementLevelA = inDataStream.FindNextElement(Context, UpperEltFound, MaxSizeToRead, AllowDummyElt);
+			inDataStream.I_O().setFilePointer(SizePosition + SizeLength, seek_beginning);
+			ElementLevelA = inDataStream.FindNextElement(sContext, UpperEltFound, MaxSizeToRead, AllowDummyElt);
 			while (ElementLevelA != NULL && MaxSizeToRead > 0 && UpperEltFound <= 0) {
 				MaxSizeToRead -= ElementLevelA->ElementSize(true); // even if it's the default value
 				if (!AllowDummyElt && ElementLevelA->IsDummy()) {
-					ElementLevelA->SkipData(inDataStream, Context);
+					ElementLevelA->SkipData(inDataStream, sContext);
 					delete ElementLevelA; // forget this unknown element
 				} else {
 					// more logical to do it afterward
@@ -430,26 +430,25 @@ void EbmlMaster::Read(EbmlStream & inDataStream, const EbmlSemanticContext & Con
 					ElementLevelA->SkipData(inDataStream, ElementLevelA->Generic().Context);
 				}
 
-				if (MaxSizeToRead <= 0) {
-					goto processCrc;// this level is finished
-		//				return; 
-				}
-				
 				if (UpperEltFound > 0) {
 					UpperEltFound--;
-					if (UpperEltFound > 0)
+					if (UpperEltFound > 0 || MaxSizeToRead <= 0)
 						goto processCrc;
-		//					return;
 					ElementLevelA = FoundElt;
 					continue;
-				} else if (UpperEltFound < 0) {
+				} 
+				
+				if (UpperEltFound < 0) {
 					UpperEltFound++;
 					if (UpperEltFound < 0)
 						goto processCrc;
-		//					return;
 				}
 
-				ElementLevelA = inDataStream.FindNextElement(Context, UpperEltFound, MaxSizeToRead, AllowDummyElt);
+				if (MaxSizeToRead <= 0) {
+					goto processCrc;// this level is finished
+				}
+				
+				ElementLevelA = inDataStream.FindNextElement(sContext, UpperEltFound, MaxSizeToRead, AllowDummyElt);
 			}
 			if (UpperEltFound > 0) {
 				FoundElt = ElementLevelA;
